from __future__ import division

import weakref
import time
import wx
from wx import aui
from M30W.debug import debug, not_implemented
from M30W import runtime
import M30W.media as media
from M30W import costume


IMAGE  = 'imageeditor'
SOUND  = 'soundeditor'
SCRIPT = 'scripteditor'


welcome_text = """
WARNING! SAVING TO .sb FILES WILL DELETE ALL SOUNDS FROM A PROJECT!

Welcome to M30W!

M30W is a program designed to allow fast
developing of Scratch projects.
It uses a unique text syntax to allow typing of
blocks rather than laggy dragging them around.

M30W currently is in development proccess.
Therefore, you cannot run scripts.
Saving to the .sb format is now implemented, so make sure to test it!

Double-click on a sprite to start edit!

Enjoy,
    the M30W dev team.

Current limitaitions:
We use kurt to parse scripts and save the .sb
files, so all its limitations apply:

- Take care with the "length of" block: strings aren't dropdowns, lists are

- length of [Hello!]      // string
- length of [list v]      // list

- Variable names (and possibly other values, such as broadcasts) can't:
  * contain special identifiers (like end, if, etc.)
  * have trailing whitespace
  * contain special characters, rather obviously: like any of []()<> or equals =
  * be named after a block, eg. a variable called "wait until"

As well as unimplemented features, e.g. such as sound saving.
They WILL come in the next days!
"""


class ScriptEditor(wx.Panel):
    font = wx.Font(10, wx.FONTFAMILY_MODERN,
                             wx.FONTSTYLE_NORMAL,
                             wx.FONTWEIGHT_NORMAL)
    style = wx.TextAttr(font=font)
    style.SetTabs([25])
    #TODO: let the user config tab width (25 * chrs)

    def __init__(self, sprite, token, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)
        self.sprite = weakref.proxy(sprite, self.DeleteMyself)
        self.token = token
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.textCtrl = wx.TextCtrl(self,
                                    style=wx.TE_MULTILINE | wx.HSCROLL)
        self.textCtrl.SetFont(self.font)
        self.textCtrl.SetDefaultStyle(self.style)
        self.textCtrl.write(self.sprite.code)

        self.sizer.Add(self.textCtrl, 1, wx.EXPAND)
        self.SetSizerAndFit(self.sizer)
        self.Bind(wx.EVT_TEXT, self.Save, self.textCtrl)
        self.lastsave = time.time()

    def Save(self, event):
        ##Allows undo&redo
        #if time.time() - self.lastletter > 0.3:
            #self.undo.append(self.textCtrl.GetValue())
        self.lastsave = time.time()
        self.sprite.code = self.textCtrl.GetValue()

    @not_implemented
    def OnRedo(self):
        pass

    @not_implemented
    def OnUndo(self):
        pass

    def DeleteMyself(self, proxy):
        leftPanel = self.GetParent().GetParent()
        wx.CallAfter(leftPanel.RemovePageByToken, self.token)


class ImageEditor(wx.Listbook):
    def __init__(self, sprite, *args, **kwargs):
        wx.Listbook.__init__(self, *args, style=wx.LB_BOTTOM)

        self.sprite = sprite
        self.il = wx.ImageList(32, 32)
        self.names = []
        self.AssignImageList(self.il)

        #object because we imported costume
        for object in self.sprite.costumes:
            self.il.Add(object.get_thumbnail(32, costume.FORMAT_BITMAP))
            self.AddPage(ResourceViewer(self,
                                        object,
                                        True),
                         object.name,
                         select=True,
                         imageId=self.il.GetImageCount() - 1
                         )

    def UpdateList(self):
        pass


class ResourceViewer(wx.Panel):
    def __init__(self, parent, resource, costume=True):
        wx.Panel.__init__(self, parent, style=wx.FULL_REPAINT_ON_RESIZE)
        self.resource = resource
        self.costume = costume

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)

        self.toolbar = wx.ToolBar(self, style=wx.TB_BOTTOM)
        self.Bind(wx.EVT_MENU,
                  self.OnCenter,
                  self.toolbar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('center'),
                                             shortHelpString="Center"))
        self.Bind(wx.EVT_MENU,
                  self.OnDuplicate,
                  self.toolbar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('duplicate'),
                                             shortHelpString="Duplicate"))
        self.toolbar.Realize()

        if self.costume:
            self.sizer.AddSpacer(10, 10)
            self.xText = wx.StaticText(self, wx.ID_ANY, label="X:")
            self.sizer.Add(self.xText, 0, border=5,
                           flag=wx.ALIGN_BOTTOM | wx.BOTTOM)

            self.xCtrl = wx.SpinCtrl(self,
                                 wx.ID_ANY,
                                 initial=self.resource.get_center()[0],
                                 min=0,
                                 max=self.resource.get_size()[0] - 1)
            self.sizer.Add(self.xCtrl, 0, flag=wx.ALIGN_BOTTOM)

            self.sizer.AddSpacer(5, 5)

            self.yText = wx.StaticText(self, wx.ID_ANY, label="Y:")
            self.sizer.Add(self.yText, 0, border=5,
                           flag=wx.ALIGN_BOTTOM | wx.BOTTOM)

            self.yCtrl = wx.SpinCtrl(self,
                                 wx.ID_ANY,
                                 initial=self.resource.get_center()[1],
                                 min=0,
                                 max=self.resource.get_size()[1] - 1)
            self.sizer.Add(self.yCtrl, 0, flag=wx.ALIGN_BOTTOM)

            self.Bind(wx.EVT_SPINCTRL, self.OnSpin, self.xCtrl)
            self.Bind(wx.EVT_SPINCTRL, self.OnSpin, self.yCtrl)

        self.sizer.Add(self.toolbar, 0,
                       flag=wx.ALIGN_BOTTOM)
        self.SetSizerAndFit(self.sizer)

        self.Bind(wx.EVT_PAINT, self.OnPaint, self)

    def OnPaint(self, event):
        #Note a VERY misleading bug when the panel is too big for the place in
        #the AUINoteBook - the costume begins to stretch over borders etc.
        size = [i - 20 for i in self.GetSize()]  # 10px padding
        size[1] -= self.toolbar.GetSize()[1]

        image_prop = self.resource.get_size()
        ratio = min(size[0] / image_prop[0], size[1] / image_prop[1])

        image_size = [int(ratio * i) for i in image_prop]
        image_pos = ((size[0] - image_size[0]) / 2 + 10,
                     (size[1] - image_size[1]) / 2 + 10)

        if image_size[0] < 1 or image_size[1] < 1:
            return event.Skip()

        dc = wx.PaintDC(self)
        dc.BeginDrawing()
        dc.DrawRectangle(image_pos[0] - 1, image_pos[1] - 1,
                         image_size[0] + 2, image_size[1] + 2)
        dc.DrawBitmap(self.resource.get_resized_image(image_size,
                                                  costume.FORMAT_BITMAP),
                      *image_pos)

        if self.costume:
            dc.DrawLine(ratio * self.xCtrl.GetValue() + image_pos[0],
                        0,
                        ratio * self.xCtrl.GetValue() + image_pos[0],
                        size[1] + 20)

            dc.DrawLine(0,
                        ratio * self.yCtrl.GetValue() + image_pos[1],
                        size[0] + 20,
                        ratio * self.yCtrl.GetValue() + image_pos[1])
        dc.EndDrawing()

        event.Skip()

    def OnCenter(self, event):
        self.xCtrl.SetValue(self.resource.get_size()[0] // 2)
        self.yCtrl.SetValue(self.resource.get_size()[1] // 2)
        self.OnSpin(event)

    @not_implemented
    def OnDuplicate(self, event):
        pass

    @not_implemented
    def OnEdit(self, event):
        pass

    def OnSpin(self, event):
        self.resource.set_center(self.xCtrl.GetValue(), self.yCtrl.GetValue())
        self.Refresh()


class LeftPanel(wx.Panel):
    def __init__(self, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)
        runtime.leftPanel = self
        self.SetMinSize((480, 500))

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        debug("Making notebook...", 1)
        self.noteBook = aui.AuiNotebook(self)
        welcome = wx.TextCtrl(self,
                            style=wx.TE_MULTILINE | wx.HSCROLL)
        welcome.write(welcome_text)
        welcome.SetFont(ScriptEditor.font)
        welcome.SetDefaultStyle(ScriptEditor.style)

        self.noteBook.AddPage(welcome, 'Welcome')
        self.welcomeOpen = True

        self.pages = []

        self.noteBook.SetMinSize((300, 450))
        debug("Done.", -1)
        self.sizer.Add(self.noteBook, 1, wx.EXPAND)
        self.SetSizerAndFit(self.sizer)
        self.SetMinSize(self.GetSize())

        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CLOSE,
                  self.OnPageClose, self.noteBook)
        #wxpython 2.9 compatibility
        if wx.VERSION[1] > 8:
            self.imageList = wx.ImageList(16, 16)
            self.noteBook.AssignImageList(self.imageList)


    def _NewPage(self, Window, title, bitmap, token):
        if self.welcomeOpen:
            self.noteBook.DeletePage(0)
            self.welcomeOpen = False
        try:
            self.noteBook.AddPage(Window, title, select=True, bitmap=bitmap)
        except TypeError:
            #API change on wxpython >= 2.9
            i = self.imageList.Add(bitmap)
            self.noteBook.AddPage(Window, title, select=True,
                                  imageId=self.imageList.GetImageCount() - 1)
            self.noteBook.SetPageImage(self.noteBook.GetPageCount() - 1, i)
        self.pages.append(token)

    def OpenScriptEditor(self, sprite):
        """OpenScriptEditor(sprite)

        Opens a new ScriptEditor tab for the given sprite.
        """
        if (sprite.name, SCRIPT) in self.pages:
            self.noteBook.SetSelection(self.pages.index((sprite.name, SCRIPT)))
            return
        self._NewPage(ScriptEditor(sprite, (sprite.name, SCRIPT), self.noteBook),
                      sprite.name, media.get_icon('script'),
                      (sprite.name, SCRIPT))

    def OpenImageEditor(self, sprite):
        """OpenImageEditor(sprite)

        Opens a new ScriptEditor tab for the given sprite.
        """
        if (sprite.name, IMAGE) in self.pages:
            self.noteBook.SetSelection(self.pages.index((sprite.name, IMAGE)))
            return
        self._NewPage(ImageEditor(sprite, self.noteBook),
                      sprite.name, media.get_icon('image'),
                      (sprite.name, IMAGE))

    @not_implemented
    def OpenSoundEditor(self, sprite):
        pass

    #Cleans self.imageList on wxpython >= 2.9
    def OnPageClose(self, event):
        if self.welcomeOpen:
            self.welcomeOpen = False
            return
        try:
            self.imageList.Remove(event.GetOldSelection())
        except AttributeError:
            pass
        self.pages.pop(event.GetOldSelection())

    def RemovePageByToken(self, token):
        index = self.pages.index(token)
        self.noteBook.DeletePage(index)

    def RefreshPages(self):
        debug('Deleting old pages...')
        try:
            old_selection = self.pages[self.noteBook.GetSelection()]
        #No open pages
        except IndexError:
            old_selection = None
        for _ in xrange(self.noteBook.GetPageCount()):
            self.noteBook.DeletePage(0)
        debug("Done.")
        
        debug("Recreating opened pages...")
        names = {sprite.name: sprite for sprite in
                 runtime.project.sprites + [runtime.project.stage]}
        functions = {SCRIPT: self.OpenScriptEditor,
                     IMAGE: self.OpenImageEditor,
                     SOUND: self.OpenSoundEditor}
        open_pages, self.pages = self.pages, []

        for name, type in open_pages:
            if name in names:
                functions[type](names[name])

        if old_selection in open_pages:
            self.noteBook.SetSelection(open_pages.index(old_selection))
        debug("Done.")



"""Provides global user settings.

:Usage:
- get_option(option, default, do_eval=False)
    Returns the value of the option specified. If non-existent returns
    default and assigns new option to it. If eval is True it returns
    evaluated answer (e.g. tuple, int, etc.)
- set_option(option, value): Sets the option specified to the value
- get_boolean(option, default):
- del_option(option): Deletes the option from the file
- save(): forces save of the configurations to the disk
"""
import sys
import os
from ConfigParser import RawConfigParser as Parser, NoOptionError


PATH = {'win32': lambda:
            os.path.join(os.environ['APPDATA'], 'M30W', 'M30W.conf'),
        'cygwin': lambda: 
            os.path.join(os.environ['APPDATA'], 'M30W', 'M30W.conf'),
        'linux2': lambda: os.path.expanduser('~/.config/M30W/M30W.conf'),
        'darwin': lambda:
            os.path.expanduser('~/Library/Application Support/M30W/M30W.conf')
        }[sys.platform]()

if not os.path.exists(PATH):
    try:
        if not os.path.isdir(os.path.dirname(PATH)):
            os.mkdir(os.path.dirname(PATH))
        open(PATH, 'w')
    except (OSError, IOError):
        raise SystemExit("Could not create config file at path '%s'" % PATH)

config = Parser()
config.read(PATH)

if not config.has_section('SETTINGS'):
    config.add_section('SETTINGS')


def get_option(option, default, do_eval=False):
    """get_option(option, default, do_eval=False) -> value

    Returns the value of the option specified. Returns default if
    non-existent and assigns new option to it. If eval is True it returns
    evaluated answer (e.g. tuple, int, etc.)
    """
    try:
        value = config.get('SETTINGS', option)
        return eval(value) if do_eval else value
    except NoOptionError:
        set_option(option, default)
        return default


def set_option(option, value):
    """set_option(option, value)

    Sets the option specified to the value specified.
    """
    config.set('SETTINGS', option, value)


def del_option(option):
    """del_option(option)

    Deletes the option from the file.
    """
    config.remove_option('SETTINGS', option)


def save():
    """save()

    Saves configurations to the config file.
    """
    with open(PATH, 'w') as config_file:
        config.write(config_file)

__all__ = (del_option, set_option, get_option, save)



import wx
from M30W.debug import debug
from M30W import runtime
from M30W import media, config

debug("Initializing right panel...", 1)
from .rightPanel import RightPanel
debug("Done.", -1)
debug("Initializing left panel...", 1)
from .leftPanel import LeftPanel
debug("Done.", -1)


class MainFrame(wx.Frame):

    def __init__(self, *args, **kwargs):
        wx.Frame.__init__(self, *args, **kwargs)
        runtime.mainFrame = self
        self.SetTitle('M30W')

        debug('Making right panel...', 1)
        self.right_pane = RightPanel(self)
        debug("Done.", -1)

        debug("Making left panel...", 1)
        self.left_pane = LeftPanel(self)
        debug("Done.", -1)

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.left_pane, 1, flag=wx.EXPAND)
        self.sizer.Add(self.right_pane, 0, flag=wx.EXPAND | wx.LC_ICON)
        self.SetSizerAndFit(self.sizer)

        self.SetMinSize(self.GetSize())

        self.SetSize(config.get_option('START_SIZE', self.GetSize(), True))
        self.Maximize(config.get_option('START_MAXIMIZED', False, True))

        self.SetIcon(media.get_icon('M30W', wx.Icon, '.ico'))
        wx.GetApp().SetTopWindow(self)
        self.Bind(wx.EVT_CLOSE, self.OnClose, self)

    def OnClose(self, event):
        debug("Saving frame size...", 1)
        config.set_option('START_SIZE', self.GetSize())
        config.set_option('START_MAXIMIZED', self.IsMaximized())

        debug("Done.", -1)
        self.Destroy()



#!/usr/bin/python
"""This is the M30W software.
Copyright (C) 2012 M30W developers.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
#M30W follows PEP8 to increase readability. Why won't you?

print """M30W Copyright (C) 2012 M30W developers.
"""
from M30W.debug import debug
from M30W.GUI import show_app

def main():
    debug("Starting GUI...", 1)
    show_app()

if __name__ == '__main__':
    main()



import wx
from M30W.debug import debug
debug("Initializing stage...", 1)
from .stage import Stage
debug("Done.", -1)
debug("Initializing sprite panel...", 1)
from .spritePanel import SpritePanel
debug("Done.", -1)


class RightPanel(wx.Panel):
    def __init__(self, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)

        self.sizer = wx.BoxSizer(wx.VERTICAL)

        debug("Making stage...", 1)
        self.stage = Stage(self)
        debug("Done.", -1)

        debug("Making sprite panel...", 1)
        self.spritePanel = SpritePanel(self)
        debug("Done.", -1)

        self.sizer.Add(self.stage, 0)
        self.sizer.Add(self.spritePanel, 1, flag=wx.EXPAND)
        self.SetSizer(self.sizer)
        self.Fit()
        self.SetMinSize((480, self.GetSize()[1]))



"""Module to store objects at runtime.
Objects are either set to None if not yet created.
GUI parts are:
* mainFrame
* leftPanel
* rightPanel
* noteBook (the coding area)
* stage
* spritePanel
Non-GUI objects are:
* sprites
"""
mainFrame = leftPanel = rightPanel = noteBook = stage = spritePanel = None

#global project, sprites, Sprite, stage
from M30W.project import Project
from M30W.sprites import Sprite
project = Project()

#Other modules should use the provided methods to change sprites.
def get_sprites():
    return tuple(project.sprites)

def get_stage():
    return project.stage

def add(sprite):
    if sprite.name in map(lambda x: x.name, project.sprites):
        raise NameError("Sprite %s already registered!" % sprite.name)
    project.sprites.append(sprite)
    spritePanel.UpdateList(len(project.sprites))


def new():
    names = map(lambda x: x.name, project.sprites)
    for i in xrange(1000):
        if 'new_sprite %s' % i in names: continue
        add(Sprite('new_sprite %s' % i))
        return project.sprites[-1]

def delete(index):
    project.sprites.pop(index)
    spritePanel.UpdateList(0)

def delete_all():
    del project.sprites[:]
    spritePanel.RecreateList(0)

def replace_all(new_sprites):
    project.sprites[:] = new_sprites
    spritePanel.RecreateList(0)

def moveup(index):
    if not index > 0:
        return
    (project.sprites[index],
     project.sprites[index - 1]) = (project.sprites[index - 1],
                                    project.sprites[index])
    spritePanel.UpdateList(index)

def movedown(index):
    if not index < len(project.sprites):
        return
    (project.sprites[index],
     project.sprites[index + 1]) = (project.sprites[index + 1],
                                    project.sprites[index])
    spritePanel.UpdateList(index + 2)



from collections import OrderedDict
from weakref import WeakKeyDictionary, proxy
import wx
from M30W.debug import debug, not_implemented
from M30W import runtime
from M30W.sprites import rotmodes
import M30W.media as media
from wx.lib.mixins.listctrl import TextEditMixin, ListCtrlAutoWidthMixin


class EditableListCtrl(wx.ListCtrl, TextEditMixin, ListCtrlAutoWidthMixin):
    def __init__(self, *args, **kwargs):
        wx.ListCtrl.__init__(self, *args, **kwargs)
        TextEditMixin.__init__(self)
        ListCtrlAutoWidthMixin.__init__(self)
        self._onResize_ = self._onResize
        self._doResize_ = self._doResize


class SpriteViewer(wx.Panel):
    sprite_attr = OrderedDict((('Name', 'name'),
              ('Position', 'pos'),
              ('Rotation', 'rotmode'),
              ('Direction', 'direction'),
              ('Draggable', 'draggable'),
              ('Visible', 'visible'),
              ('Volume', 'volume')))
    stage_attr = OrderedDict((('Name', 'name'),
             ('Volume', 'volume'),
             ('Tempo (BPM)', 'tempo')))

    def __init__(self, parent, sprite):
        super(SpriteViewer, self).__init__(parent)
        self.sprite = proxy(sprite, lambda proxy: self.Destroy())

        self.listCtrl = EditableListCtrl(self, style=wx.LC_REPORT |
                                wx.LC_NO_HEADER | wx.LC_VRULES | wx.LC_HRULES)
        self.listCtrl.InsertColumn(0, 'property')
        self.listCtrl.InsertColumn(1, 'value')
        if self.sprite.name == 'Stage':
            self.sprite_attr = self.stage_attr
        for label, attr in self.sprite_attr.iteritems():
            self.listCtrl.Append((label, getattr(self.sprite, attr)))
        self.listCtrl.SetColumnWidth(0, -1)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.listCtrl, 1, wx.EXPAND)
        self.SetSizerAndFit(self.sizer)

        self.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBeginLabelEdit,
                  self.listCtrl)
        self.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.OnEndLabelEdit,
                  self.listCtrl)

    def __repr__(self):
        return '<SpriteViewer for sprite "%s" at %s>' % (self.sprite.name,
                                                         id(self))

    def OnBeginLabelEdit(self, event):
        event.Allow()
        attr = self.sprite_attr[self.listCtrl.GetItem(event.GetIndex(),
                                                      0).GetText()]
        if event.m_col == 0: event.Veto()
        #Don't let Stage get renamed
        if self.sprite.name == 'Stage' and attr == 'name':
            event.Veto()
        if attr in self.specials:
            self.listCtrl.SetStringItem(event.GetIndex(), 1,
                                    str(self.specials[attr](event.GetLabel())))
            setattr(self.sprite, attr, self.specials[attr](event.GetLabel()))
            event.Veto()


    def OnEndLabelEdit(self, event):
        attr = self.sprite_attr[self.listCtrl.GetItem(event.GetIndex(),
                                                      0).GetText()]
        value = event.GetLabel()
        try:
            setattr(self.sprite, attr, value)
            self.listCtrl.SetStringItem(event.GetIndex(), 1,
                                        getattr(self.sprite, attr))
        except (TypeError, ValueError):
            self.listCtrl.SetStringItem(event.GetIndex(), 1,
                                        str(getattr(self.sprite, attr)))
            event.Veto()

    specials = {'draggable': lambda t: False if t == 'True' else True,
                'visible': lambda t: False if t == 'True' else True,
                'rotmode':
            lambda t: rotmodes[(rotmodes.index(t) + 1) % 3]}

    def AllowSizing(self):
        self.listCtrl._onResize = self.listCtrl._onResize_
        self.listCtrl._doResize = self.listCtrl._doResize_

    def DisableSizing(self):
        self.listCtrl._onResize = lambda: None
        self.listCtrl._doResize = lambda: None


class SpritePanel(wx.Panel):
    def __init__(self, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)
        runtime.spritePanel = self

        debug("Making sprites listbook...")

        self.listBook = wx.Listbook(self, style=wx.LB_LEFT)
        self.il = wx.ImageList(32, 32)
        self.listBook.AssignImageList(self.il)
        #Stores refs to SpriteViewers and thumbnails
        self.last_update = WeakKeyDictionary()
        self.UpdateList()
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnOpenScriptEditor,
                  self.listBook.GetListView())

        debug("Done.")

        debug("Making Toolbar...")
        self.toolBar = wx.ToolBar(self, style=wx.TB_HORIZONTAL)
        self.Bind(wx.EVT_MENU, self.OnNew,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_NEW),
                                shortHelpString="Make New Sprite"))
        self.Bind(wx.EVT_MENU, self.OnOpen,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_FILE_OPEN),
                                shortHelpString="New Sprite From File"))
        self.Bind(wx.EVT_MENU, self.OnExport,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_FILE_SAVE),
                                shortHelpString="Export Sprite"))
        self.Bind(wx.EVT_MENU, self.OnDelete,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_CROSS_MARK),
                                shortHelpString="Delete Sprite"))
        self.Bind(wx.EVT_MENU, self.OnDown,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_GO_DOWN),
                                shortHelpString="Move Down"))
        self.Bind(wx.EVT_MENU, self.OnUp,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_GO_UP),
                                shortHelpString="Move Up"))

        self.toolBar.AddSeparator()

        self.Bind(wx.EVT_MENU,
                  self.OnOpenScriptEditor,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('script'),
                                             shortHelpString="Edit Scripts"))
        self.Bind(wx.EVT_MENU,
                  self.OnOpenImageEditor,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('image'),
                                             shortHelpString="Edit Costumes"))
        self.Bind(wx.EVT_MENU,
                  self.OnOpenSoundEditor,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('sound'),
                                             shortHelpString="Edit Sounds"))

        self.toolBar.Realize()

        debug("Done.")

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.listBook, 1, flag=wx.EXPAND)
        self.sizer.Add(self.toolBar, 0, flag=wx.EXPAND)
        self.SetSizerAndFit(self.sizer)

    @staticmethod
    def GetItems():
        return (runtime.get_stage(),) + runtime.get_sprites()

    @staticmethod
    def ShortenName(name, max_len):
        if len(name) < max_len: return name
        else: return name[:max_len - 3] + '...'

    #I tried to optimize it really hard - it still takes around 0.3 seconds
    #when you have about ~30 sprites, which is not the fastest :/
    def UpdateList(self, select=0):
        mapping = {}
        for sprite, (panel, thumbnail) in self.last_update.iteritems():
            mapping[sprite] = (panel, thumbnail)
            panel.DisableSizing()

        debug("Removing pages...")
        self.listBook.Freeze()
        while self.listBook.GetPageCount() != 0:
            self.listBook.RemovePage(0)
        self.il.RemoveAll()
        debug("Done.")

        for sprite in self.GetItems():
            if not sprite in mapping:
                mapping[sprite] = (SpriteViewer(self.listBook, sprite),
                            sprite.costumes[sprite.costume].get_thumbnail(32))
            self.il.Add(mapping[sprite][1])

            mapping[sprite][0].DisableSizing()
            self.listBook.AddPage(mapping[sprite][0],
                                  self.ShortenName(sprite.name, 10),
                                  imageId=self.il.GetImageCount() - 1
                                  )

        for sprite, (panel, thumbnail) in mapping.iteritems():
            self.last_update[sprite] = panel, thumbnail
            panel.AllowSizing()
        self.listBook.SetSelection(select)
        self.listBook.Thaw()

    @not_implemented
    def UpdateThumbnail(self, index):
        pass

    def OnOpenScriptEditor(self, event):
        runtime.leftPanel.OpenScriptEditor(
                            self.GetItems()[self.listBook.GetSelection()])

    def OnOpenImageEditor(self, event):
        runtime.leftPanel.OpenImageEditor(
                            self.GetItems()[self.listBook.GetSelection()])

    @not_implemented
    def OnOpenSoundEditor(self, event):
        pass

    def OnNew(self, event):
        runtime.new()

    @not_implemented
    def OnOpen(self, event):
        pass

    @not_implemented
    def OnExport(self, event):
        pass

    def OnDelete(self, event):
        if self.listBook.GetSelection() == 0:
            return
        runtime.delete(self.listBook.GetSelection() - 1)

    def OnDown(self, event):
        if self.listBook.GetSelection() == self.listBook.GetPageCount() - 1:
            return
        runtime.movedown(self.listBook.GetSelection() - 1)

    def OnUp(self, event):
        if self.listBook.GetSelection() < 2:
            return
        runtime.moveup(self.listBook.GetSelection() - 1)



import time
from M30W.debug import debug, not_implemented
from M30W.costume import DEFAULT_COSTUME
try:
    import kurt
except ImportError:
    pass

class Base(object):
    kwords = {'costumes': lambda: [],
              'costume': lambda: 0,
              'code': lambda: "",
              'vars': lambda: {},
              'lists': lambda: {},
              'volume': lambda: 100}

    def __init__(self, name, **kwargs):
        """Base(self, name, costumes=[], costume=0, code="", vars={}, lists={},
                volume=100)

        :Parameters:
        - `name`, `costumes`,`costume`, `code`, `vars`,
          `lists`, `volume`
        """
        debug("Creating sprite %s" % name)
        self._name = name

        # Functions because shared mutable objects
        for kword in self.kwords:
            if kword in kwargs: setattr(self, '_' + kword, kwargs[kword])
            else: setattr(self, '_' + kword, self.kwords[kword]())

        if not self.costumes:
            self.costumes.append(DEFAULT_COSTUME)

    def __repr__(self):
        return "<Sprite '%s' at %s>" % (self.name, id(self))

    @classmethod
    def from_kurt(cls, morph):
        raise NotImplementedError

    def to_kurt(self):
        raise NotImplementedError
    
    #Called from children, to reduce code be rewritten.
    def set_kurt_attrs(self, morph):
        morph.images = [costume.to_kurt() for costume in self._costumes]
        morph.costume = morph.images[self._costume]
        lists = {}
        for name, list in self._lists.iteritems():
            lists[name] = kurt.ScratchListMorph(name=name, items=list)
        morph.lists = lists
        morph.vars = self._vars.copy()  # I don't trust kurt :)
        morph.volume = self._volume

        code = unicode(self.code).split('\n' * 3)
        code = [i for i in code if i.strip("\r\n ")]
        for sheet in code:
            sheet = kurt.parse_block_plugin(sheet)
            sheet.morph = morph
            sheet.pos = kurt.Point([20, 20])
            morph.scripts.extend([sheet])


    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def code(self):
        return self._code

    @code.setter
    def code(self, value):
        self._code = value

    @property
    def vars(self):
        return self._vars

    @property
    def lists(self):
        return self._lists

    @property
    def costumes(self):
        return self._costumes

    @property
    def costume(self):
        return self._costume

    @costume.setter
    def costume(self, value):
        self.costumes[value]  # Validate that index is in range
        self._custume = value

    @property
    def volume(self):
        return self._volume

    @volume.setter
    def volume(self, value):
        value = float(value)
        if not 0 <= value <= 100:
            print value
            raise TypeError("Value must bew in range 0-100")
        self._volume = value


    # Sounds support comes later.

    #==========================================================================
    # Begin blocks code
    #==========================================================================
    @not_implemented
    def forever(self, args, cargs):
        pass

    @not_implemented
    def repeat(self, args, cargs):
        pass

    @not_implemented
    def dountil(self, args, crags):
        pass

    @not_implemented
    def doif(self, args, cargs):
        pass

    @not_implemented
    def doifelse(self, args, crags):
        pass

    def wait(self, args):
        exec_time = time.time()
        while not exec_time - time.time() > args[0]:
            yield

    @not_implemented
    def waituntil(self, args):
        pass



from M30W.parser import get_spec


class Block():
    def __init__(self, code, cargs):
        """__init__(code (Line object), cargs) -> Block object

        Returns a new Block object."""
        self.cargs = cargs
        self.code = code
        self.sprite = code.sprite
        self.line = code.line

        self.spec = get_spec(self.code)
        if not self.spec:
            raise SyntaxError("""SyntaxError at line %s in sprite %s.
Could not parse %s.""" % (self.line + 1, self.sprite, self.code))
        self.args = self.spec.get_args(self.code)

    def __call__(self):
        """__call__()

        Executes this block (NOT queuing up)
        Returns when finished running (or never)"""
        args = []
        #If inputs are reporters
        for arg in self.args:
            if not arg.__class__ == Block:
                args.append(arg)
            else:
                args.append(arg())
        try:
            getattr(self.sprite, self.handler)(args=args, cargs=self.cargs)
        except Exception:
            raise RuntimeError('Caught runtime exception at line %s.')



"""Copyright (C) 2012 M30W developers

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import traceback
from M30W.debug import debug, DEBUG
from M30W import config
from M30W import runtime
#Trying to import wx, raising an error message using Tkinter if not found.
try:
    debug("Importing wxPython...", 1)
    debug("Ensuring version 2.8...")
    #import wxversion
    #wxversion.ensureMinimal('2.8')
    debug("Importing wx...")
    import wx
    debug("Done.", -1)
except ImportError:
    from Tkinter import *
    import tkMessageBox
    root = Tk()
    root.withdraw()
    print "No wxPython found! Please install it from wxpython.org!"
    tkMessageBox.showerror("No wx found!",
                            "Please install it from wxpython.org!")
    sys.exit("No wxPython module was found on this computer!")

#Enable for debugging through Ctrl+Alt+I
#from wx.lib.mixins.inspection import InspectableApp
#class App(InspectableApp):
class App(wx.App):
    def __init__(self):
        wx.App.__init__(self, False)
        self.SetAppName("M30W")
        self._org_except_hook = sys.excepthook
        def excepthook(type, obj, tb):
            dlg = wx.MessageDialog(None, "M30W encountered an error:\n%s"
                                         "\nDo You want to abort?"
                                         % '\n'.join(
                                traceback.format_exception(type, obj, tb)
                                                     ),
                                   "M30W encountered an error!",
                                   wx.ICON_ERROR | wx.CANCEL | wx.OK)
            if dlg.ShowModal() == wx.ID_OK:
                self.GetTopWindow().Close()
                self._org_except_hook(type, obj, tb)

        if DEBUG: sys.excepthook = excepthook
    def CreateGUI(self):
        self.Bind(wx.EVT_CLOSE, self.OnExit)

        debug("Making main frame...", 1)
        self.mainFrame = MainFrame(None)
        debug('Done.', -1)

        debug("Making menus...", 1)
        menubar = MenuBar(self.mainFrame, menus)
        self.mainFrame.SetMenuBar(menubar)
        debug("Done.", -1)

        debug('Showing main frame...')
        self.mainFrame.Show(True)
        debug("Done.")
    def OnExit(self):
        config.save()


def init_app():
    debug("Initializing menus...", 1)
    global menus, MenuBar, MainFrame
    from .menu import menus, MenuBar
    debug("Done.", -1)
    debug("Initializing main frame...", 1)
    from .mainFrame import MainFrame
    debug('Done.', -1)


def show_app():
    #Creating app here to avoid wx._core.PyNoAppError
    app = App()

    debug("Initializing app...", 1)
    init_app()
    debug("Done.", -1)

    debug("Creating GUI...", 1)
    app.CreateGUI()
    debug("Done.", -1)

    debug("Starting main loop...")
    app.MainLoop()



from .base import Base
from M30W.costume import Costume
from M30W.lists import ScratchList
try:
    import kurt
except ImportError:
    pass


class Stage(Base):
    kwords = Base.kwords.copy()
    kwords.update({'tempo': lambda: 60})

    def __init__(self, **kwargs):
        """Stage(self, costumes=[], costume=0, code="", vars={}, lists={},
                volume=100, tempo=60)

        :Parameters:
        - `costumes`,`costume`, `code`, `vars`, `lists`, `volume`, `tempo`
        """
        Base.__init__(self, 'Stage', **kwargs)

    @classmethod
    def from_kurt(cls, morph):
        code = ("\n" * 3).join([script.to_block_plugin()
                                    for script in morph.scripts])
        costumes = []
        for costume in morph.backgrounds:
            costumes.append(Costume(costume.get_image(),
                                    costume.name,
                                    costume.rotationCenter.value))
        costume = morph.backgrounds.index(morph.background)
        lists = {name: ScratchList(*list.items) for
                 name, list in morph.lists.iteritems()}
        obj = cls(costumes=costumes,
                   costume=costume,
                   code=code,
                   vars=morph.variables.copy(),
                   lists=lists,
                   volume=morph.volume,
                   tempo=morph.tempoBPM)
        obj.scripts = morph.scripts
        return obj

    def to_kurt(self):
        morph = kurt.Stage()
        self.set_kurt_attrs(morph)
        morph.tempoBPM = self._tempo
        return morph

    @property
    def tempo(self):
        return self._tempo

    @tempo.setter
    def tempo(self, value):
        self._tempo = value



from .base import Base
from M30W.costume import Costume
from M30W.lists import ScratchList
try:
    import kurt
except ImportError:
    pass


NORMAL = 'normal'
RL = "leftRight"
NO = "none"


class Sprite(Base):
    kwords = Base.kwords.copy()
    kwords.update({'x': lambda: 0.0,
                   'y': lambda: 0.0,
                   'direction': lambda: 90.0,
                   'rotmode': lambda: NORMAL,
                   'draggable': lambda: False,
                   'size': lambda: 100.0,
                   'visible': lambda: True})

    def __init__(self, name, **kwargs):
        super(Sprite, self).__init__(name, **kwargs)

    @classmethod
    def from_kurt(cls, morph):
        costumes = [Costume.from_kurt(costume) for costume in morph.costumes]
        costume = morph.costumes.index(morph.costume)
        x, y, _, _ = morph.bounds.value
        y *= -1
        rx, ry = costumes[costume].get_center()
        x += rx - 240
        y += -ry + 180
        code = ("\n" * 3).join([script.to_block_plugin()
                                for script in morph.scripts])
        rotmode = morph.rotationStyle.value
        direction = (morph.rotationDegrees + 90.0) % 360
        if direction > 180: direction -= 360
        lists = {name: ScratchList(*list.items) for
                 name, list in morph.lists.iteritems()}
        return cls(morph.name,
                   costumes=costumes,
                   costume=costume,
                   code=code,
                   vars=morph.variables.copy(),
                   lists=lists,
                   volume=morph.volume,
                   x=x,
                   y=y,
                   direction=direction,
                   rotmode=rotmode,
                   draggable=morph.draggable,
                   size=morph.scalePoint.value[0] * 100,
                   visible=not morph.flags)


    def to_kurt(self):
        morph = kurt.Sprite()
        self.set_kurt_attrs(morph)
        morph.name = self.name
        morph.flags = 0 if self.visible else 1
        morph.rotationStyle = kurt.Symbol(self.rotmode)
        direction = self.direction
        if direction < 0: direction += 360
        direction = (direction - 90) % 360
        morph.rotationDegrees = direction
        morph.scalePoint = kurt.Point((self.size / 100,) * 2)
        rx, ry = self.costumes[self.costume].get_center()
        x1, y1 = self.x, self.y
        x1 -= rx - 240
        y1 += ry - 180
        y1 *= -1
        w, h = self.costumes[self.costume].get_size()
        x2, y2 = x1 + w, y1 + h
        morph.bounds = kurt.Rectangle([x1, y1, x2, y2])

        return morph

    @property
    def x(self):
        return self._x

    @x.setter
    def x(self, value):
        self._x = float(value)

    @property
    def y(self):
        return self._y

    @y.setter
    def y(self, value):
        self._y = float(value)

    @property
    def pos(self):
        return '%s:%s' % (self.x, self.y)

    @pos.setter
    def pos(self, value):
        self.x, self.y = [float(x) for x in value.split(':')]

    @property
    def direction(self):
        return self._direction

    @direction.setter
    def direction(self, value):
        self._direction = float(value)

    @property
    def rotmode(self):
        return self._rotmode

    @rotmode.setter
    def rotmode(self, value):
        if not value in (NORMAL, RL, NO):
            raise TypeError("Invalid mode!")
        self._rotmode = value

    @property
    def draggable(self):
        return self._draggable

    @draggable.setter
    def draggable(self, value):
        self._draggable = bool(value)

    @property
    def size(self):
        return self._size

    @size.setter
    def size(self, value):
        self._size = float(value)

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, value):
        self._visible = bool(value)



from distutils.core import setup
import sys

long_description = \
"""
M30W is a program designed to allow fast
developing of Scratch projects.
It uses a unique text syntax to allow typing of
blocks rather than laggy dragging them around.
"""

scripts = ['scripts/M30W']
if 'bdist_wininst' in sys.argv:
    scripts = ['scripts/M30W_post_install.py']

setup(name="M30W",
      version="0.0.15",
      maintainer="roijac",
      maintainer_email="roi.jacoboson1@gmail.com",
      url="http://scratch.mit.edu/forums/viewtopic.php?id=106225",
      install_requires=['kurt', 'PIL', 'wxpython'],
      packages=['M30W', 'M30W', 'M30W.compiler', 'M30W.GUI',
                'M30W.parser', 'M30W.sprites'],
      package_data={'M30W': ['icons/*', 'COPYING.txt']},
      scripts=scripts,
      description="""GUI text-based interface to Scratch (by MIT) projects.""",
      long_description=long_description,
      license="GPL3")



import re


class Spec():
    """This class provides a OOP interface to parsing blocks.

    Spec arguments are:
    %s    is string, like in 'say %s'
    %n    is number, like in '%n + %n'
    %b    is boolean, like in 'wait until %b'
          because you can't really put inputs inside it,
          it accepts only nested reporters
    %c    is a c block, like in 'forever %c'
    %m    stands for many, because it accepts numbers as
          well as strings like in '%m > %m'.
          It is made because while in GUI there's no
          difference between number and string writing,
          in M30W you have to either write numbers as is
          or put strings inside apostrophes
    """
    args = {letter: '(' + pattern + '|\[.*\])' for letter, pattern in
            {'%s': r"'.*(?<!\\)'",
            '%n': r'\-?[0-9.]+',
            '%c': r'{script}',
            '%b': r'\[.*\]', # Because it can only match nested blocks.
            '%m': r"\-?[0-9.]+|'.*(?<!\\)'"}.items()}

    def __init__(self, spec, handler):
        """__init___(spec, handler) -> Spec object

        Creates new Spec object"""
        self.spec = re.escape(spec)  # Escaping characters
        self.spec = self.spec.replace('\\%', '%')  # We don't need them escaped
        self.handler = handler
        #Allows multiple whitespace characters
        #self.spec = self.spec.replace(' ', '\s+')  # Breaks up the matching :/
        #Replacing %(input) with equivalent re pattern
        for letter, pattern in self.args.items():
            self.spec = self.spec.replace(letter, pattern)

    def matches(self, key):
        """match(key) -> bool

        Check if the key matches one of the block specs."""
        return bool(re.match(self.spec, key))

    def handler(self):
        """handler() -> string

        Get the block handler as a string."""
        return self.handler

    def get_args(self, code):
        """get_args(code) -> list

        Returns the arguments that were caught using regex."""
        from M30W.compiler import Block  # Because of recursive imports.
        args = list(re.match(self.spec, code).groups())
        for index, arg in enumerate(args):
            if arg[0] + arg[-1] == '[]':
                args[index] = Block(arg[1:-1], [])
            if arg[0] + arg[-1] == "''":
                args[index] = arg[1:-1].replace("\\'", "'")
        return args

from specshandler import specs


def get_spec(key):
    for spec in specs:
        if spec.matches(key):
            return spec
    return None



"""Handles media and makes them available platform-independent.
"""
import os.path
import wx

M30W_FOLDER = os.path.split(__file__)[0]

class _Provider():
    def __init__(self, path):
        self.path = path

    def __call__(self, name, format=wx.Bitmap, ext='.png'):
        if format == wx.Bitmap:
            return wx.Bitmap(os.path.join(self.path, name + ext),
                              wx.BITMAP_TYPE_ANY)
        if format == wx.Image:
            return wx.Image(os.path.join(self.path, name + ext),
                              wx.BITMAP_TYPE_ANY)
        if format == wx.Icon:
            return wx.Icon(os.path.join(self.path, name + ext),
                              wx.BITMAP_TYPE_ANY)

get_icon = _Provider(os.path.join(M30W_FOLDER, 'icons'))



import json
import os


from specs import Spec

specspath = os.path.join(*os.path.split(__file__)[:-1] + ('specs.mspecs',))

rawspecs = json.load(open(specspath))

specs = []
for spec in rawspecs:
    specs.append(Spec(spec["spec"], spec["command"]))


def add():
    name = raw_input("Name?")
    spec = raw_input("Spec?")
    command = raw_input("Command?")
    blocktype = raw_input("Type?")
    while blocktype not in ["stack", "reporter"]:
        blocktype = raw_input("Invalid type! (stack/reporter/cap)\nType:")
    rawspecs.append({'name': name, 'spec': spec, 'command': command,
                  'type': blocktype})


def save():
    with open(specspath, 'w') as file:
        json.dump(rawspecs, file, indent=2, sort_keys=True)


if __name__ == "__main__":
    for spec in rawspecs:
        print spec["spec"], "executes", spec["command"]
    while raw_input("Continue? (y)") in ["y", "Y"]:
        add()
    save()



"""
Project class - stores sprites, project infos etc.
"""
import cPickle as pickle
from weakref import WeakValueDictionary as WVD
from M30W import sprites
from M30W.debug import debug, not_implemented, ensure_kurt
try:
    import kurt
    from M30W.costume import _convert, FORMAT_PIL
except ImportError:
    kurt = None


class NoSavePath(Exception): pass


class Project(object):
    kwords = {'sprites': lambda: [],
              'stage': sprites.Stage,
              'author': lambda: u'',
              'comment': lambda: u'',
              'path': lambda: None}

    def __init__(self, **kwargs):
        for kword in self.kwords:
            if kword in kwargs: setattr(self, '_' + kword, kwargs[kword])
            else: setattr(self, '_' + kword, self.kwords[kword]())
        self.last_save = (WVD({i: s for i, s in enumerate(self.sprites)}),
                          self.stage,
                          self.author,
                          self.comment,
                          self.path)

    @classmethod
    def from_kurt(cls, morph):
        stage = sprites.Stage.from_kurt(morph.stage)
        #Redefining sprites from outer scope...
        
        sprite = [sprites.Sprite.from_kurt(i) for i in morph.stage.sprites]
        author = morph.info['author']
        comment = morph.info['comment']
        path = morph.path
        return cls(stage=stage,
                   sprites=sprite,
                   author=author,
                   comment=comment,
                   path=path)

    def to_kurt(self):
        morph = kurt.ScratchProjectFile(self.path, load=False)
        morph.stage = self.stage.to_kurt()
        morph.sprites = [sprite.to_kurt() for sprite in self.sprites]
        for sprite in morph.sprites:
            for script in sprite.scripts:
                script.replace_sprite_refs(morph.get_sprite)
        from M30W import runtime
        thumbnail = _convert(runtime.stage.GetBitmap(), FORMAT_PIL)
        thumbnail = thumbnail.resize((160, 120))

        #We don't need the name, so we leave it empty
        morph.thumbnail = kurt.Image.from_image("", thumbnail)
        morph.info['author'] = self.author
        morph.info['comment'] = self.comment
        return morph

    @classmethod
    @not_implemented
    def from_sb2(cls, obj):
        pass

    @not_implemented
    def to_sb2(self):
        pass

    def save(self):
        debug("Saving project to %s" % self.path, 1)
        if self.path.endswith('.sb2'):
            raise NotImplementedError
        elif self.path.endswith('.m30w'):
            pickle.dump(self, open(self.path, 'wb'), protocol=2)
        #Change default to .sb2 when Scratch comes out?
        else:
            ensure_kurt()
            self.to_kurt().save()
        debug("Done.", -1)

    def load(self):
        debug("Loading project from %s" % self.path, 1)
        if not self.path:
            raise ValueError("Project not saved yet!")
        if self.path.endswith('.sb2'):
            raise NotImplementedError
        elif self.path.endswith('.m30w'):
            project = pickle.load(self, open(self.path, 'wb'), protocol=2)
            self.__dict__ = Project.from_kurt(project).__dict__
        #Change default to .sb2 when Scratch comes out?
        else:
            ensure_kurt()
            project = kurt.ScratchProjectFile(self.path)
            self.__dict__ = Project.from_kurt(project).__dict__
        debug("Done.", -1)

    @property
    def changed(self):
        changed = filter(None, [sprite.changed for
                                sprite in self.sprites + [self.stage]])
        changed = changed or self.last_save != (WVD({i: s for i, s in
                                                     enumerate(self.sprites)}),
                                                self.stage,
                                                self.author,
                                                self.comment,
                                                self.path)
        return bool(changed)

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, value):
        self._path = value

    @property
    def sprites(self):
        return self._sprites

    @property
    def stage(self):
        return self._stage

    @stage.setter
    def stage(self, value):
        assert isinstance(value, sprites.Stage)
        self._stage = value

    @property
    def author(self):
        return self._author

    @author.setter
    def author(self, value):
        self._author = unicode(value)

    @property
    def comment(self):
        return self._comment

    @comment.setter
    def comment(self, value):
        self._comment = unicode(value)



"""
Provides IO functions
"""
import M30W.runtime as runtime
from M30W.debug import debug

def save():
    runtime.project.save()

def save_as(path):
    old_path = runtime.project.path
    try:
        runtime.project.path = path
        save()
    except Exception:
        runtime.project.path = old_path
        raise

def open(path):
    old_path = runtime.project.path
    try:
        runtime.project.path = path
        reload()
    except Exception:
        runtime.project.path = old_path
        raise

def reload():
    #Referencing to old sprites, so panels won't be garbage-collected
    refs = [runtime.project.stage] + runtime.project.sprites

    runtime.project.load()
    debug("Updating SpritePanel...", 1)
    runtime.spritePanel.UpdateList()
    debug("Done.", -1)
    debug("Refreshing stage...", 1)
    runtime.stage.Refresh()
    debug("Done.", -1)
    debug("Refreshing the notebook...", 1)
    runtime.leftPanel.RefreshPages()
    debug("Done.", -1)



import wx
from M30W import runtime


#Need to switch to canvas.
class Stage(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        runtime.stage = self
        self.SetSizeHints(480, 360, 480, 360)
        self.SetBackgroundColour(wx.WHITE)
        self.SetSize((480, 360))
        self.Bind(wx.EVT_PAINT, self.OnPaint, self)

    def OnPaint(self, event):
        dc = wx.PaintDC(self)
        dc.BeginDrawing()
        stage = runtime.project.stage
        dc.DrawBitmap(stage.costumes[stage.costume].get_image(), 0, 0)
        dc.EndDrawing()

    def GetBitmap(self):
        context = wx.ClientDC( self )
        memory = wx.MemoryDC( )
        x, y = self.ClientSize
        bitmap = wx.EmptyBitmap( x, y, -1 )
        memory.SelectObject( bitmap )
        memory.Blit( 0, 0, x, y, context, 0, 0)
        memory.SelectObject( wx.NullBitmap)
        return bitmap


"""Scratch lists - data structures to hold basic data.
"""


class ScratchList():
    def __init__(self, *items):
        """ScratchList(*items) -> List object

        Create a new ScratchList object and optionally assign
        its items.
        This is pretty much the same as a normal Python list
        and has the same interface except the index offset
        (starts at 1) and the fact that negative indexes aren't valid.
        """
        self.items = list(items)

    def __repr__(self):
        return '<ScratchList: %s>' % repr(self.items)

    def __len__(self):
        return len(self.items)

    def __getitem__(self, key):
        if key < 1:
            raise IndexError
        return self.items[key + 1]

    def __setitem__(self, key, value):
        if key < 1:
            raise IndexError
        self.items[key + 1] = value

    def __delitem__(self, key):
        if key < 1:
            raise IndexError
        del self.items[key]

    def __contains__(self, item):
        return item in self.items

    def __iter__(self):
        return iter(self.items)

    def append(self, value):
        self.items.append(value)

    def delall(self):
        self.items = []

    def insert(self, index, value):
        self.items.insert(index, value)
